diff --git a/lists.lisp b/lists.lisp
index 00b42fa..a6daa4e 100644
--- a/lists.lisp
+++ b/lists.lisp
@@ -341,15 +341,20 @@ Example:
                             one)))))
     (%map-product (ensure-function function) (cons list more-lists))))
 
-(defun flatten (tree)
+(defun flatten (tree &key to-depth)
   "Traverses the tree in order, collecting non-null leaves into a list."
   (let (list)
-    (labels ((traverse (subtree)
+    (labels ((traverse (subtree cur-depth)
                (when subtree
-                 (if (consp subtree)
-                     (progn
-                       (traverse (car subtree))
-                       (traverse (cdr subtree)))
-                     (push subtree list)))))
-      (traverse tree))
+                 (cond ((and (consp subtree)
+			     (integerp to-depth)
+			     (>= cur-depth to-depth))
+			(push (car subtree) list)
+			(traverse (cdr subtree) cur-depth))
+		       ((consp subtree)
+			(traverse (car subtree) (1+ cur-depth))
+			(traverse (cdr subtree) cur-depth))
+		       (t
+			(push subtree list))))))
+      (traverse tree 0))
     (nreverse list)))
diff --git a/tests.lisp b/tests.lisp
index 7ffe30f..2daa779 100644
--- a/tests.lisp
+++ b/tests.lisp
@@ -850,6 +850,22 @@
     (flatten '((1) 2 (((3 4))) ((((5)) 6)) 7))
   (1 2 3 4 5 6 7))
 
+(deftest flatten.2
+    (flatten '((1) 2 (((3 4))) ((((5)) 6)) 7) :to-depth 1)
+  (1 2 ((3 4)) (((5)) 6) 7))
+
+(deftest flatten.3
+    (flatten '((1) 2 (((3 4))) ((((5)) 6)) 7) :to-depth 2)
+  (1 2 (3 4) ((5)) 6 7))
+
+(deftest flatten.4
+    (flatten '((1) 2 (((3 4))) ((((5)) 6)) 7) :to-depth 3)
+  (1 2 3 4 (5) 6 7))
+
+(deftest flatten.5
+    (flatten '((1) 2 (((3 4))) ((((5)) 6)) 7) :to-depth 4)
+  (1 2 3 4 5 6 7))
+
 (deftest remove-from-plist.1
     (let ((orig '(a 1 b 2 c 3 d 4)))
       (list (remove-from-plist orig 'a 'c)
